<?php


namespace app\models\generator;


use yii\web\HttpException;

class EasyPackMakerAPI
{
    private $api_url;
    private $userName;
    private $apiKey;
    private $status;
    private $error;
    private $response;


    public function __construct($userName, $apiKey)
    {
        $this->api_url = 'https://easypackmaker.com/generator/api';
        $this->userName = $userName;
        $this->apiKey = $apiKey;
    }

    public function __get($name)
    {
        switch ($name) {
            case 'error':
                return $this->error;
            case 'response':
                return htmlentities($this->response);
            default:
                if ($this->response) {
                    if ($json = json_decode($this->response, true)) {
                        foreach ($json as $key => $value) {
                            if (strtolower($name) == strtolower($key)) {
                                return $json[$key];
                            }
                        }
                    }
                }
                return false;
        }
    }

    /**
     * Builds a query string and call sendRequest method.
     * Could be used to custom API call method.
     *
     * @param mixed $args query params
     *
     * @return mixed
     * @throws HttpException
     */
    public function generate($args)
    {
        $url = $this->api_url;
        if (is_array($args)) {
            if (!array_key_exists('UserName', $args)) {
                $args['UserName'] = $this->userName;
            }
            if (!array_key_exists('Token', $args)) {
                $args['Token'] = $this->_genToken($args);
            }
        }
        return $this->_sendRequest($url, $args);
    }

    /**
     * @param $response
     * @return bool|false|string
     */
    public function decodePdf($response) {
        $modelArray = json_decode($response, true);
        if ($modelArray["ErrorCode"] === 0) {
            return base64_decode($modelArray["Model"]);
        }

        return false;
    }

    private function _genToken($args)
    {
        $args['Password'] = $this->apiKey;
        ksort($args);

        $token = '';
        foreach ($args as $arg) {
            if (!is_array($arg)) {
                if ($arg === true) {
                    $arg = 'true';
                } elseif ($arg === false) {
                    $arg = 'false';
                }
                $token .= (string) $arg;
            }
        }
        $token = hash('sha256', $token);

        return $token;
    }

    /**
     * Main method. Call API with params
     *
     * @param $api_url
     * @param $args
     * @return bool|string
     * @throws HttpException
     */
    private function _sendRequest($api_url, $args)
    {
        $this->error = '';
        if (is_array($args)) {
            $args = json_encode($args);
        }

        if ($curl = curl_init()) {
            curl_setopt($curl, CURLOPT_URL, $api_url);
            curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $args);
            curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
            ));

            $out = curl_exec($curl);
            $this->response = $out;
            $json = json_decode($out);

            if ($json) {
                if (@$json->ErrorCode !== "0") {
                    $this->error = @$json->Details;
                } else {
                    $this->status = @$json->Status;
                }
            }

            curl_close($curl);

            return $out;

        } else {
            throw new HttpException('Can not create connection to ' . $api_url . ' with args ' . $args, 404);
        }
    }
}